unit Unit1; 

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, Grids;

const
  ciMinesCount = 10;
  
type

  { TfMain }

  TfMain = class(TForm)
    sgMain: TStringGrid;
    procedure FormCreate(Sender: TObject);
    procedure sgMainClick(Sender: TObject);
    procedure sgMainDrawCell(Sender: TObject; aCol, aRow: Integer;
      aRect: TRect; aState: TGridDrawState);
  private
    { private declarations }
  public
    { public declarations }
    procedure PrepareField;
    function CountMines (x, y: integer): integer;
    function CountClosedCells: integer;
    procedure RecurseField (x, y: integer);
  end;

var
  fMain: TfMain;

implementation

{ TfMain }

procedure TfMain.FormCreate(Sender: TObject);
begin
  PrepareField;
end;

procedure TfMain.sgMainClick(Sender: TObject);
var
  lStatus: integer;
  lOpened: boolean;
begin
  with sgMain do begin
    if Pos ('*', Cells [Col, Row]) = 0 then begin
      lOpened := (Cells [Col, Row][2] = '1');
      if lOpened then
        Exit;
      lStatus := StrToInt (Cells [Col, Row][1]);
      if lStatus = 0 then
        RecurseField (Col, Row)
      else
        Cells [Col, Row] := IntToStr (lStatus) + '1';
    end else begin
      ShowMessage ('You''ve just blowed up!');
      PrepareField;
    end;
  end;
  Invalidate;
  if CountClosedCells = ciMinesCount then begin
    ShowMessage ('You''ve open all cells!');
    PrepareField;
  end;
end;

procedure TfMain.sgMainDrawCell(Sender: TObject; aCol, aRow: Integer;
  aRect: TRect; aState: TGridDrawState);
var
  lOpened: boolean;
  lStatus: integer;
begin
  with sgMain do begin
    lOpened := (Cells [ACol, ARow][2] = '1');
    if lOpened then begin
      lStatus := StrToInt (Cells [ACol, ARow][1]);
      with Canvas do begin
        Brush.Color := clWhite;
        FillRect(ARect);
        case lStatus of
          0: Font.Color := clNone;
          1: Font.Color := clGreen;
          2: Font.Color := clYellow
          else
            Font.Color := clRed;
        end;
        TextOut(ARect.Left + 2, AREct.Top + 2, IntToStr (lStatus));
      end;
    end else
      with Canvas do begin
        Brush.Color := clNavy;
        FillRect(ARect);
      end;
  end;
end;

procedure TfMain.PrepareField;
var
  x, y, i: integer;
begin
  for x := 0 to ciMinesCount - 1 do
    for y := 0 to ciMinesCount - 1 do
      sgMain.Cells [x, y] := '';
  Randomize;
  for i := 1 to ciMinesCount do begin
    repeat
      x := Random (10);
      y := Random (10);
    until sgMain.Cells [x, y] = '';
    sgMain.Cells [x, y] := '*0';
  end;
  for y := 1 to ciMinesCount do
    for x := 1 to ciMinesCount do begin
      if Pos ('*', sgMain.Cells [x - 1, y - 1]) = 0 then
        sgMain.Cells [x - 1, y - 1] := IntToStr (CountMines (x, y)) + '0';
    end;
end;

function TfMain.CountMines(x, y: integer): integer;
var
  i, j: integer;
begin
  Result := 0;
  for i := 1 to ciMinesCount do
    for j := 1 to ciMinesCount do begin
      if (Pos ('*', sgMain.Cells [i - 1, j - 1]) > 0)
        and (Abs (x - i) <= 1) and (Abs (y - j) <= 1) then
        Inc (Result);
    end;
end;

function TfMain.CountClosedCells: integer;
var
  x, y: integer;
begin
  Result := 0;
  for x := 0 to ciMinesCount - 1 do
    for y := 0 to ciMinesCount - 1 do
      if sgMain.Cells [x, y][2] = '0' then
        Inc (Result);
end;

procedure TfMain.RecurseField(x, y: integer);
var
  i, j: integer;
begin
  with sgMain do begin
    if Cells [x, y][2] = '1' then
      Exit;
    Cells [x, y] := Cells [x, y][1] + '1';
    for i := 0 to ciMinesCount - 1 do
      for j := 0 to ciMinesCount - 1 do
        if (Abs (x - i) <= 1) and (Abs (y - j) <= 1) then begin
          if (x = i) and (y = j) then
            Continue;
          if Cells [x, y][1] = '0' then
            RecurseField (i, j);
        end;
  end;
end;

initialization
  {$I unit1.lrs}

end.

